@@ -29,6 +29,8 @@ gp_globals$base_url = "http://biit.cs.ut.ee/gprofiler"
 #' If 'multi_query' is selected, the result is a data frame for comparing multiple input lists,
 #' just as in the web tool.
 #'
+#' The input gene lists are not stored in g:Profiler unless the option 'as_short_link' is set to TRUE.
+#'
 #' @param query character vector, or a (named) list of character vectors for multiple queries, that can consist of mixed types of gene IDs (proteins, transcripts, microarray IDs, etc), SNP IDs, chromosomal intervals or term IDs.
 #' @param organism organism name. Organism names are constructed by concatenating the first letter of the name and the
 #' family name. Example: human - 'hsapiens', mouse - 'mmusculus'.
@@ -55,16 +57,17 @@ gp_globals$base_url = "http://biit.cs.ut.ee/gprofiler"
 #'  MIRNA, CORUM, HP, HPA, WP. Please see the g:GOSt web tool for the comprehensive
 #'  list and details on incorporated data sources.
 #' @param as_short_link indicator to return results as short-link to the g:Profiler web tool. If set to TRUE, then the function returns the results URL as a character string instead of the data.frame.
+#' @param highlight indicator to return a TRUE-FALSE column called 'highlighted' to indicate driver terms in GO.
 #' @return A named list where 'result' contains data.frame with the enrichment analysis results and 'meta' contains metadata needed for Manhattan plot. If the input
 #'  consisted of several lists the corresponding list is indicated with a variable
-#'  'query'.
+#'  'query'. The 'result' data.frame is ordered first by the query name, data source (such as GO:BP, GO:CC, GO:MF, REAC, etc), and then by the adjusted p-value.
 #'  When requesting a 'multi_query', either TRUE or FALSE, the columns of the resulting data frame differ.
 #'  If 'evcodes' is set, the return value includes columns 'evidence_codes' and 'intersection'.
 #'  The latter conveys info about the intersecting genes between the corresponding query and term.
 #'
 #'  The result fields are further described in the \href{https://cran.r-project.org/package=gprofiler2/vignettes/gprofiler2.html}{vignette}.
 #'
-#'  If 'as_short_link' is set to TRUE, then the result is a character short-link to see and share corresponding results via the g:Profiler web tool.
+#'  If 'as_short_link' is set to TRUE, then the result is a character short-link to see and share corresponding results via the g:Profiler web tool. In this case, the input gene lists will be stored in a database.
 #' @author  Liis Kolberg <liis.kolberg@@ut.ee>, Uku Raudvere <uku.raudvere@@ut.ee>
 #' @examples
 #' gostres <- gost(c("X:1000:1000000", "rs17396340", "GO:0005005", "ENSG00000156103", "NLRP1"))
@@ -84,18 +87,44 @@ gost <- function(query,
                       custom_bg = NULL,
                       numeric_ns  = "",
                       sources = NULL,
-                      as_short_link = FALSE
+                      as_short_link = FALSE,
+                      highlight = FALSE
                     ) {
 
   url = paste0(file.path(gp_globals$base_url, "api", "gost", "profile"), "/")
-
+  
+  # get data_version
+  version_info = try(gprofiler2::get_version_info(), silent = TRUE)
+  if("try-error" %in% class(version_info)){
+    base_url = gprofiler2::get_base_url()
+    # get data version from archive
+    data_version = basename(base_url)
+  }else{
+    data_version = version_info$gprofiler_version
+  }
+  # extract ensembl version
+  ensembl_version = as.numeric(gsub("\\D", "", strsplit(data_version, "_")[[1]][1]))
+  if (highlight & ensembl_version < 108){
+    message("Note that the set g:Profiler version doesn't support GO term highlighting")
+  }
+  if (highlight & ordered_query){
+    message("Highlighting is not supported for ordered query. Setting 'highlight = FALSE'")
+    highlight = FALSE
+  }
+  if (highlight & !significant){
+    message("Highlighting is not supported if all results are returned (significant = FALSE). Setting 'highlight = FALSE'")
+    highlight = FALSE
+  }
+  
   # Query
 
   if (is.null(query) | all(is.na(query))) {
-    stop("Missing query")
+    message("Missing query")
+    return(NULL)
   } else if (is.list(query)) {
     if (is.data.frame(query)){
-      stop("Query can't be a data.frame. Please use a vector or list of identifiers.")
+      message("Query can't be a data.frame. Please use a vector or list of identifiers.")
+      return(NULL)
     }
     # Multiple queries
     qnames = names(query)
@@ -122,6 +151,9 @@ gost <- function(query,
 
   if (startsWith(organism, "gp__")){
     message("Detected custom GMT source request")
+    if (highlight){
+      message("Highlighting option doesn't work with custom GMT data")
+    }
     if (!is.null(sources)){
       message("Sources selection is not available for custom GMT requests. All sources in the GMT upload will be used.")
       sources <- NULL
@@ -134,7 +166,8 @@ gost <- function(query,
 
   if (!is.null(custom_bg)){
     if (!is.vector(custom_bg)){
-      stop("custom_bg must be a vector")
+      message("custom_bg must be a vector")
+      return(NULL) 
     }
     if (!domain_scope %in% c("custom_annotated", "custom")){
       message("Detected custom background input, domain scope is set to 'custom'")
@@ -143,7 +176,8 @@ gost <- function(query,
     t <- ifelse(length(custom_bg) == 1, custom_bg <- jsonlite::unbox(custom_bg), custom_bg <- custom_bg)
   }else{
     if (domain_scope %in% c("custom_annotated", "custom")){
-      stop("Domain scope is set to custom, but no background genes detected from the input.")
+      message("Domain scope is set to custom, but no background genes detected from the input.")
+      return(NULL) 
     }
   }
 
@@ -158,37 +192,11 @@ gost <- function(query,
     }
 
     url = file.path("http://biit.cs.ut.ee", "gplink", "l")
-
-    body <- jsonlite::toJSON((
-      list(
-        url = jsonlite::unbox(file.path(gprofiler2::get_base_url(), "gost")),
-        payload = {
-          list(
-            organism = jsonlite::unbox(organism),
-            query = jsonlite::unbox(query2),
-            sources = sources,
-            user_threshold = jsonlite::unbox(user_threshold),
-            all_results = jsonlite::unbox(!significant),
-            ordered = jsonlite::unbox(ordered_query),
-            no_evidences = jsonlite::unbox(!evcodes),
-            combined = jsonlite::unbox(multi_query),
-            measure_underrepresentation = jsonlite::unbox(measure_underrepresentation),
-            no_iea = jsonlite::unbox(exclude_iea),
-            domain_scope = jsonlite::unbox(domain_scope),
-            numeric_ns = jsonlite::unbox(numeric_ns),
-            significance_threshold_method = jsonlite::unbox(correction_method),
-            background = custom_bg)
-        }
-      )
-    ),
-    auto_unbox = FALSE,
-    null = "null")
-
-  } else {
-    body <- jsonlite::toJSON((
+    
+    payload = {
       list(
         organism = jsonlite::unbox(organism),
-        query = query,
+        query = jsonlite::unbox(query2),
         sources = sources,
         user_threshold = jsonlite::unbox(user_threshold),
         all_results = jsonlite::unbox(!significant),
@@ -200,21 +208,58 @@ gost <- function(query,
         domain_scope = jsonlite::unbox(domain_scope),
         numeric_ns = jsonlite::unbox(numeric_ns),
         significance_threshold_method = jsonlite::unbox(correction_method),
-        background = custom_bg,
-        output = jsonlite::unbox("json")
+        background = custom_bg)
+    }
+    # add driver terms parameter starting from ensembl version 108
+    if (ensembl_version >= 108 & !startsWith(organism, "gp__")) {
+      payload$highlight <- jsonlite::unbox(highlight)
+    }
+    body <- jsonlite::toJSON((
+      list(
+        url = jsonlite::unbox(file.path(gprofiler2::get_base_url(), "gost")),
+        data_version = jsonlite::unbox(data_version),
+        payload = payload
       )
     ),
     auto_unbox = FALSE,
     null = "null")
-  }
 
+  } else {
+    payload = list(
+      organism = jsonlite::unbox(organism),
+      query = query,
+      sources = sources,
+      user_threshold = jsonlite::unbox(user_threshold),
+      all_results = jsonlite::unbox(!significant),
+      ordered = jsonlite::unbox(ordered_query),
+      no_evidences = jsonlite::unbox(!evcodes),
+      combined = jsonlite::unbox(multi_query),
+      measure_underrepresentation = jsonlite::unbox(measure_underrepresentation),
+      no_iea = jsonlite::unbox(exclude_iea),
+      domain_scope = jsonlite::unbox(domain_scope),
+      numeric_ns = jsonlite::unbox(numeric_ns),
+      significance_threshold_method = jsonlite::unbox(correction_method),
+      background = custom_bg,
+      output = jsonlite::unbox("json")
+    )
+    # add driver terms parameter starting from ensembl version 108
+    if (ensembl_version >= 108 & !startsWith(organism, "gp__")) {
+      payload$highlight <- jsonlite::unbox(highlight)
+    }
+    body <- jsonlite::toJSON((
+      payload
+    ),
+    auto_unbox = FALSE,
+    null = "null")
+  }
+  
   # Headers
   headers <-
     list("Accept" = "application/json",
          "Content-Type" = "application/json",
          "charset" = "UTF-8",
-          "Expect" = "")
-oldw <- getOption("warn")
+         "Expect" = "")
+  oldw <- getOption("warn")
   options(warn = -1)
   h1 = RCurl::basicTextGatherer(.mapUnicode = FALSE)
   h2 = RCurl::getCurlHandle() # Get info about the request
@@ -236,9 +281,11 @@ oldw <- getOption("warn")
   txt <- h1$value()
 
   if (rescode != 200) {
-    stop("Bad request, response code ", rescode)
+    message("The g:Profiler web resource you're trying to connect to is not available at the moment or has changed.")
+    message("Please check your internet connection or try again later, or contact us on biit.support@ut.ee")
+    return(NULL) 
   }
-
+  
   res <- jsonlite::fromJSON(txt)
 
   if (as_short_link){
@@ -288,9 +335,7 @@ oldw <- getOption("warn")
       "term_name",
       "effective_domain_size",
       "source_order",
-      "parents"
-    )
-
+      "parents")
     if (evcodes) {
       col_names <- append(col_names, c("evidence_codes", "intersection"))
       # Add column that lists the intersecting genes separated by comma
@@ -300,19 +345,22 @@ oldw <- getOption("warn")
           genes <- meta$genes_metadata$query[[query]]$ensgs[which(lengths(evcodes) > 0)]
           genes2 <- lapply(genes, function(x) ifelse(x %in% genemap, names(which(genemap == x)) , x))
           return(paste0(genes2, collapse = ","))
-          },
+        },
         df$intersections,
         df$query,
         SIMPLIFY = TRUE
       )
       df$evidence_codes <- sapply(df$intersections, function(x)
-          paste0(sapply(x[which(lengths(x) > 0)], paste0, collapse = " "), collapse = ","), USE.NAMES = FALSE)
+        paste0(sapply(x[which(lengths(x) > 0)], paste0, collapse = " "), collapse = ","), USE.NAMES = FALSE)
     }
-
+  
     # Order by query, source and p_value
     df <- df[with(df, order(query, source, p_value)), ]
 
   }
+  if (highlight & ensembl_version >= 108 & !startsWith(organism, "gp__")){
+    col_names <- append(col_names, "highlighted")
+  }
 
   # Rename native to term_id
   colnames(df)[colnames(df) == "native"] <- "term_id"
@@ -657,10 +705,10 @@ publish_gosttable <- function(gostres, highlight_terms = NULL, use_colors = TRUE
   # gostres is the GOSt response list (contains results and metadata) or a data frame
   term_id <- p_values <- query <- p_value <- NULL
 
-  if (class(gostres) == "list"){
+  if (is.list(gostres) & !is.data.frame(gostres)){
     if (!("result" %in% names(gostres))) stop("Name 'result' not found from the input")
     df <- gostres$result
-  } else if (class(gostres) == "data.frame"){
+  } else if (is.data.frame(gostres)){
     df <- gostres
   } else {
     stop("The input 'gostres' should be a data frame or a list from the gost() function.")
@@ -920,7 +968,9 @@ upload_GMT_file <- function(gmtfile){
     rescode = RCurl::getCurlInfo(h2)[["response.code"]]
 
     if (rescode != 200) {
-      stop("Bad request, response code ", rescode)
+      message("The g:Profiler web resource you're trying to connect to is not available at the moment or has changed.")
+      message("Please check your internet connection or try again later, or contact us on biit.support@ut.ee")
+      return(NULL) 
     }
 
     txt <- h1$value()
@@ -941,7 +991,9 @@ upload_GMT_file <- function(gmtfile){
     rescode = RCurl::getCurlInfo(h2)[["response.code"]]
 
     if (rescode != 200) {
-      stop("Bad request, response code ", rescode)
+      message("The g:Profiler web resource you're trying to connect to is not available at the moment or has changed.")
+      message("Please check your internet connection or try again later, or contact us on biit.support@ut.ee")
+      return(NULL) 
     }
     res <- jsonlite::fromJSON(r)
   }
@@ -1001,7 +1053,9 @@ oldw <- getOption("warn")
   txt <- h1$value()
 
   if (rescode != 200) {
-    stop("Bad request, response code ", rescode)
+    message("The g:Profiler web resource you're trying to connect to is not available at the moment or has changed.")
+    message("Please check your internet connection or try again later, or contact us on biit.support@ut.ee")
+    return(NULL) 
   }
   res <- jsonlite::fromJSON(txt)
   return(res)
@@ -1096,7 +1150,9 @@ gconvert = function(
   txt <- h1$value()
 
   if (rescode != 200) {
-    stop("Bad request, response code ", rescode)
+    message("The g:Profiler web resource you're trying to connect to is not available at the moment or has changed.")
+    message("Please check your internet connection or try again later, or contact us on biit.support@ut.ee")
+    return(NULL) 
   }
 
   res <- jsonlite::fromJSON(txt)
@@ -1112,7 +1168,11 @@ gconvert = function(
 
   # filter empty results
   if (filter_na){
-    df <- df[df$converted != "None",]
+    df <- df[!df$converted %in% c("nan", "None"),]
+  }else{
+    # convert to NA-s
+    df[df == "nan"] <- NA_character_
+    df[df == "None"] <- NA_character_
   }
   if (nrow(df) == 0){
     message("No results to show\n", "Please make sure that the organism or namespace is correct")
@@ -1235,7 +1295,9 @@ gorth <- function(
   txt <- h1$value()
 
   if (rescode != 200) {
-    stop("Bad request, response code ", rescode)
+    message("The g:Profiler web resource you're trying to connect to is not available at the moment or has changed.")
+    message("Please check your internet connection or try again later, or contact us on biit.support@ut.ee")
+    return(NULL) 
   }
 
   res <- jsonlite::fromJSON(txt)
@@ -1356,7 +1418,9 @@ gsnpense <- function(
   txt <- h1$value()
 
   if (rescode != 200) {
-    stop("Bad request, response code ", rescode)
+    message("The g:Profiler web resource you're trying to connect to is not available at the moment or has changed.")
+    message("Please check your internet connection or try again later, or contact us on biit.support@ut.ee")
+    return(NULL) 
   }
 
   res <- jsonlite::fromJSON(txt)
@@ -1473,3 +1537,61 @@ set_base_url = function(url) {
   assign("base_url", url, envir = gp_globals)
 }
 
+#' Get version info of g:Profiler data sources
+#'
+#' @param organism organism name. Organism names are constructed by concatenating the first letter of the name and the family name. Example: human - 'hsapiens', mouse - 'mmusculus'.
+#' @return A named nested list that includes the versions for all the data sources (GO, KEGG, Reactome, WP, etc) at the time of the data extraction for the given organism.
+#' The versions correspond to the g:Profiler version embedded in the base_url which is also returned by this function under the name 'gprofiler_version'.
+#' @author Liis Kolberg <liis.kolberg@@ut.ee>
+#' @examples
+#' \dontrun{version_info <- get_version_info(organism = "hsapiens")}
+#'
+#' @export
+get_version_info <- function(organism = "hsapiens"){
+  url = file.path(gprofiler2::get_base_url(), "api", "util", "data_versions")
+
+  body <- jsonlite::toJSON((
+    list(
+      organism = jsonlite::unbox(organism)
+    )
+  ),
+  auto_unbox = FALSE,
+  null = "null")
+
+  # Headers
+  headers <-
+    list("Accept" = "application/json",
+         "Content-Type" = "application/json",
+         "charset" = "UTF-8",
+         "Expect" = "")
+  oldw <- getOption("warn")
+  options(warn = -1)
+  h1 = RCurl::basicTextGatherer(.mapUnicode = FALSE)
+  h2 = RCurl::getCurlHandle() # Get info about the request
+
+  # Request
+  r = RCurl::curlPerform(
+    url = url,
+    postfields = body,
+    httpheader = headers,
+    customrequest = 'POST',
+    verbose = FALSE,
+    ssl.verifypeer = FALSE,
+    writefunction = h1$update,
+    curl = h2,
+    .opts = gp_globals$rcurl_opts
+  )
+  options(warn = 0)
+  rescode = RCurl::getCurlInfo(h2)[["response.code"]]
+  txt <- h1$value()
+
+  if (rescode != 200) {
+    message("The g:Profiler web resource you're trying to connect to is not available at the moment or has changed.")
+    message("Please check your internet connection or try again later, or contact us on biit.support@ut.ee")
+    return(NULL) 
+  }
+
+  version_info <- jsonlite::fromJSON(txt)
+
+  return(version_info)
+}
